/*
 * Copyright (c) 2004-2009  DMC Co., Ltd.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 */
/*
 * EUC
 * tab=4
 */
#include <linux/kernel.h>
#include <linux/slab.h>
#include <linux/poll.h>
#include <linux/usb.h>
#include <linux/version.h>
#if (LINUX_VERSION_CODE < KERNEL_VERSION(3,0,0))
#include <linux/smp_lock.h>
#endif
#include <linux/init.h>
#include <linux/module.h>
#include "../vers.h"
#include "usbdmc.h"

#define	SUB_VERS		""

#define DEBUG_HON		0

#define	EEPROM_EMPTY_CHECK	1
#define	EEPROM_TSC20_BUG	1

#define USBDMC_MINOR	181		/* minor */
#define VID_USBDMC		0x0afa	/* vendor ID */
#define PID_USBDMC		0x03e8	/* product ID */

#define	true	1
#define	false	0

#define	INI_2_IDLE		0x05
#define	S_RATE			0x40
#define	RESET			0x55
#define	IDLE_2_XYP		0x31
#define	IDLE_2_XYP2		0x01
#define	IDLE_2_XYP3		0x21
#define	XYP_2_IDLE		0x02
#define	GETSTAT			0x15
#define	IDLE_2_RomRd	0x1d
#define	IDLE_2_RomWr	0x0d
#define	MemWr			0x19
#define	MemRd			0x18
#define	ACK				0x06
#define	NAK				0x15

#define	TMV_NORM		(HZ*2)
#define	TMV_EEPROM		(HZ*5)
//#define	TMV_NORM		(HZ*10)
//#define	TMV_EEPROM		(HZ*10)

typedef struct
{
	struct semaphore	sem_entry;
    int					isopen;
	int					inum;
    struct usb_device	*dev;				/* usb device data */
#if KER26
    struct urb			*irq;				/* URB for interrupt      */
#else
    struct urb			irq;				/* URB for interrupt      */
#endif
	int errno;
    wait_queue_head_t	wait;				/* wait queue for interrupt */
    wait_queue_head_t	remove_ok;   
    int	probe_ok;   
#define	PROBE_NONE		0
#define	PROBE_AVAIL		0x12345678
	int					mode;
	int					samp_rate;
	char				data[64];
	int					rxputp;
	int					rxgetp;
#define	MAX_RXBUF	256
    char				rxbuf[MAX_RXBUF];	/* recv buffer */
    int					ep_intr_in;			/* ENDPOINT(INT-IN)*/
    int					ep_bulk_out;
    int					ep_bulk_in;
	char				svd[5];

	int					offdelay;
	int					req_offtimer_timeup;
	char				offd[5];
	struct timer_list 	tmlist;
	int					testrsv[3];
} usbdmc_data_t;

#define	GET_INTERFACE_NUM(dev) 	(dev->descriptor.iProduct==0?0:1)
#define	MAXIF	2
static usbdmc_data_t *g_usbdmc_data0;
static usbdmc_data_t *g_usbdmc_data1;

static void usbdmc_offtimer(unsigned long arg);
static void pktdump(char *title,int len,char *p,int inum);
static int usbdmc_setup(usbdmc_data_t* pdata);
static int usbdmc_reset(usbdmc_data_t* pdata);
static int usbdmc_specify(usbdmc_data_t* pdata,int rate);
static int usbdmc_writemem(usbdmc_data_t* pdata,int addr,int data);
static int usbdmc_readmem(usbdmc_data_t* pdata,int addr,int *data);
static int usbdmc_goto_idle(usbdmc_data_t* pdata);
static int usbdmc_goto_xyp(usbdmc_data_t* pdata);
static int usbdmc_get_stat(usbdmc_data_t* pdata,u_short *stat);
static int usbdmc_read_eeprom(usbdmc_data_t* pdata,u_char *buf);
static int usbdmc_write_eeprom(usbdmc_data_t* pdata,u_char *buf);
static int usbdmc_open(struct inode *inode, struct file *file);
static int usbdmc_close(struct inode *inode, struct file *file);
#if HAVE_UNLOCKED_IOCTL
static int usbdmc_ioctl(struct file *file,unsigned int cmd, unsigned long arg);
#else
static int usbdmc_ioctl(struct inode *inode, struct file *file,
               unsigned int cmd, unsigned long arg);
#endif
static ssize_t usbdmc_read(struct file * file, char * buffer, size_t count, loff_t *ppos);
static unsigned int usbdmc_poll(struct file *file, poll_table * wait);
#if KER26
static int usbdmc_probe(struct usb_interface *intface,const struct usb_device_id *id);
static void usbdmc_disconnect(struct usb_interface *intface);
#else
static void* usbdmc_probe(struct usb_device *dev, unsigned int ifnum,
                          const struct usb_device_id *id);
static void usbdmc_disconnect(struct usb_device *dev, void *ptr);
#endif

/*
 int usb_control_msg(struct usb_device *dev, unsigned int pipe,
			   	__u8 request, __u8 requesttype,
			   	__u16 value, __u16 index, void *data, __u16 size, int timeout)
 [sample]
	usb_control_msg(dev, usb_snddefctrl(dev), USB_REQ_SET_ADDRESS,
		0, dev->devnum, 0, NULL, 0, HZ * GET_TIMEOUT);
	usb_control_msg(dev, usb_rcvctrlpipe(dev, 0),
			USB_REQ_GET_DESCRIPTOR, USB_DIR_IN,
			(type << 8) + index, 0, buf, size, HZ * GET_TIMEOUT)
*/
static void pktdump(char *title,int len,char *p,int inum)
{
int i,k;
char s[256];
unsigned char *pp;
	k = sprintf(s,"DMC(%d):%s : len=%d",inum,title,len);
	if( len <= 0 )
	{
		if( len < 0 )
			printk(KERN_ERR "%s\n",s);
		return;
	}
	k += sprintf(&s[k]," data=");
	if( p )
	{
		for( pp=(unsigned char *)p,i=0 ; i < len ; i++ )
		{
			k += sprintf(&s[k],"$%02x ",pp[i]);
		}
	}
	printk(KERN_ERR "%s\n",s);
}
static int set_mode(usbdmc_data_t *pdata,int cmd)
{
	printk(KERN_ERR "DMC(%d) set_mode %02x\n",pdata->inum,cmd);
	pdata->mode = cmd;
	return(0);
}
static int usbdmc_setup(usbdmc_data_t* pdata)
{
u_short st;
	/* usbdmc_reset(pdata); */
	usbdmc_specify(pdata,0);
	usbdmc_goto_xyp(pdata);
	usbdmc_get_stat(pdata,&st);
	return(true);
}
static int usbdmc_reset(usbdmc_data_t* pdata)
{
int res;
char bf[10];
	pdata->errno = 0;
	res = usb_control_msg(pdata->dev,usb_rcvctrlpipe(pdata->dev,0),RESET,0xc0,0,0,bf,2,TMV_NORM);
	pktdump("usbdmc_reset",res,bf,pdata->inum);
	if( res > 0 && bf[0] == ACK )
	{
		set_mode(pdata,RESET);
		return(true);	// ACK goto init mode
	}
	pdata->errno = res < 0 ? res : -EIO;
	return(false);		// NAK
}
static int usbdmc_specify(usbdmc_data_t* pdata,int rate)
{
int res,a;
char bf[10];
	pdata->errno = 0;
	if( rate == 6 )
		a = 0x10;
	else if( rate >= 0 && rate <= 5 )
		a = rate;
	else
	{
		pdata->errno = -EIO;
		return(false);
	}
	pdata->samp_rate = rate;
	res = usb_control_msg(pdata->dev,usb_rcvctrlpipe(pdata->dev,0),INI_2_IDLE,0xc0,S_RATE+a,0,bf,2,TMV_NORM);
	//pktdump("usbdmc_specify",res,bf,pdata->inum);
	printk(KERN_ERR "DMC(%d):rate=%d %d res=%d\n",pdata->inum,rate,a,res);
#if EEPROM_EMPTY_CHECK
	if( res > 0 )
	{
		set_mode(pdata,INI_2_IDLE);
		return(true);   // reply recv goto idle mode
	}
#else
	if( res > 0 && bf[0] == ACK )
	{
		set_mode(pdata,INI_2_IDLE);
		return(true);	// ACK goto idle mode
	}
#endif
	pdata->errno = res < 0 ? res : -EIO;
	return(false);		// NAK
}
static int usbdmc_writemem(usbdmc_data_t* pdata,int addr,int data)
{
int res;
u_short a[2];
char bf[10];
	pdata->errno = 0;
	a[0] = (u_short)addr;
	a[1] = (u_short)data;
	printk(KERN_ERR "DMC(%d):usbdmc_writemem $%04x=$%04x\n",pdata->inum,a[0],a[1]);
	res = usb_control_msg(pdata->dev,usb_rcvctrlpipe(pdata->dev,0)
			,MemWr,0xc0,a[1],a[0],bf,2,TMV_NORM);
	pktdump("usbdmc_writemem",res,bf,pdata->inum);
	if( res > 0 && bf[0] == ACK )
		return(true);	// ACK goto idle mode
	pdata->errno = res < 0 ? res : -EIO;
	return(false);		// NAK
}
static int usbdmc_readmem(usbdmc_data_t* pdata,int addr,int *data)
{
int res;
u_short a[2];
u_char bf[10];
	pdata->errno = 0;
	a[0] = (u_short)addr;
	printk(KERN_ERR "DMC(%d):usbdmc_readmem $%04x\n",pdata->inum,a[0]);
	res = usb_control_msg(pdata->dev,usb_rcvctrlpipe(pdata->dev,0),
			MemRd,0xc0,0,a[0],bf,2,TMV_NORM);
	pktdump("usbdmc_readmem",res,bf,pdata->inum);
	if( res < 0 )
	{
		pdata->errno = res;
		return(false);		// error
	}
	*data = (bf[0]<<8) + bf[1];
	return(true);	// 
}
static int usbdmc_goto_idle(usbdmc_data_t* pdata)
{
int res;
char bf[10];
	pdata->errno = 0;
	switch( pdata->mode )
	{
	case	IDLE_2_XYP:
	case	IDLE_2_XYP2:
	case	IDLE_2_XYP3:
		break;
	default:
		return(true);
	}
	res = usb_control_msg(pdata->dev,usb_rcvctrlpipe(pdata->dev,0),XYP_2_IDLE,0xc0,0,0,bf,2,TMV_NORM);
	pktdump("usbdmc_goto_idle",res,bf,pdata->inum);
	if( res > 0 && bf[0] == ACK )
	{
		set_mode(pdata,XYP_2_IDLE);
		return(true);	// ACK goto xyp mode
	}
	pdata->errno = res < 0 ? res : -EIO;
	return(false);		// NAK
}
static int usbdmc_goto_xyp(usbdmc_data_t* pdata)
{
int res;
u_char bf[10];
	pdata->errno = 0;
	switch( pdata->mode )
	{
	case	IDLE_2_XYP:
	case	IDLE_2_XYP2:
	case	IDLE_2_XYP3:
		res = usb_control_msg(pdata->dev,usb_rcvctrlpipe(pdata->dev,0),XYP_2_IDLE,0xc0,0,0,bf,2,TMV_NORM);
		break;
	}
	res = usb_control_msg(pdata->dev,usb_sndctrlpipe(pdata->dev,0),IDLE_2_XYP,0x40,0,0,NULL,0,TMV_NORM);
	pktdump("usbdmc_goto_xyp",res,NULL,pdata->inum);
	set_mode(pdata,IDLE_2_XYP);
	return(true);	// goto xyp mode
}
static int usbdmc_get_stat(usbdmc_data_t* pdata,u_short *stat)
{
int res;
u_char bf[10];
	pdata->errno = 0;
	switch( pdata->mode )
	{
	case	IDLE_2_XYP:
	case	IDLE_2_XYP2:
	case	IDLE_2_XYP3:
		res = usb_control_msg(pdata->dev,usb_rcvctrlpipe(pdata->dev,0),XYP_2_IDLE,0xc0,0,0,bf,2,TMV_NORM);
		//printk(KERN_ERR "DMC(%d):usbdmc_get_stat s\n",pdata->inum);
		mdelay(100);
		set_mode(pdata,XYP_2_IDLE);
		break;
	}
	res = usb_control_msg(pdata->dev,usb_rcvctrlpipe(pdata->dev,0),GETSTAT,0xc0,0,0,bf,2,TMV_NORM);
	pktdump("usbdmc_get_stat",res,bf,pdata->inum);
	if( res <= 0 )
	{
		pdata->errno = res;
		return(false);
	}
	*stat = (bf[0]<<8) + bf[1];
	return(true);
}
static int usbdmc_read_eeprom(usbdmc_data_t* pdata,u_char *bf)
{
int res,i,cnt,k;
u_short st;
	pdata->errno = 0;
	switch( pdata->mode )
	{
	case	IDLE_2_XYP:
	case	IDLE_2_XYP2:
	case	IDLE_2_XYP3:
		res = usb_control_msg(pdata->dev,usb_rcvctrlpipe(pdata->dev,0),XYP_2_IDLE,0xc0,0,0,bf,2,TMV_NORM);
		set_mode(pdata,XYP_2_IDLE);
		break;
	}
	if( !usbdmc_get_stat(pdata,&st) )
		return(-1);
	printk(KERN_ERR "DMC(%d):usbdmc_read_eeprom $%x\n",pdata->inum,st);
#if EEPROM_EMPTY_CHECK
    if( (st & 0x00ff) != 0x00ff && st & 0x0080 )
        return(-1);
#else
	if( st & 0x0080 )
		return(-1);
#endif
	mdelay(100);
	res = usb_control_msg(pdata->dev,usb_rcvctrlpipe(pdata->dev,0),IDLE_2_RomRd,0xc0,0,0,bf,2,TMV_EEPROM);
    printk(KERN_ERR "DMC(%d);usbdmc_read_eeprom read res=%d\n",pdata->inum,res);
	//pktdump("usbdmc_read_eeprom",res,bf,pdata->inum);
#if EEPROM_EMPTY_CHECK
    if( res <= 0 )
    {
#if EEPROM_TSC20_BUG
        if( (st & 0x00ff) == 0x00ff && res != -EPIPE )
		{
			pdata->errno = res;
            return(-1);
		}
#endif
        bf[0] = bf[1] = 0;
        return(0);  /* eeprom empty */
    }
#else
	if( res <= 0 )
	{
		pdata->errno = res;
		return(-1);
	}
#endif
	cnt = (5*5)+1;
	for( k=2,i=0 ; i < (cnt-1) ; i++,k+=8 )
	{
		mdelay(100);
		res = usb_control_msg(pdata->dev, usb_rcvctrlpipe(pdata->dev, 0),
				IDLE_2_RomRd, 0xc0, 0, 0, &bf[k], 8, TMV_EEPROM);
		//pktdump("usbdmc_read_eeprom",res,&bf[k],pdata->inum);
		if( res <= 0 )
		{
			pdata->errno = res;
	//		return(-1);
		}
	}
	mdelay(100);
	res = usb_control_msg(pdata->dev, usb_rcvctrlpipe(pdata->dev, 0),
			IDLE_2_RomRd, 0xc0, 0, 0, &bf[k], 4, TMV_EEPROM);
	//pktdump("usbdmc_read_eeprom",res,&bf[k],pdata->inum);
	k+=4;
	if( pdata->errno )
	{
		printk(KERN_ERR "DMC(%d):usbdmc_read_eeprom err %d\n",pdata->inum,pdata->errno);
		return(-1);
	}
	printk(KERN_ERR "DMC(%d):usbdmc_read_eeprom res %d\n",pdata->inum,k);
	return(k);
}
static int usbdmc_write_eeprom(usbdmc_data_t* pdata,u_char *bf)
{
int i,res;
char s[20];
u_short *p,a;
	pdata->errno = 0;
	for( i=0,p=(u_short *)bf ; i < 39 ; i++ )
	{
		mdelay(100);
		a = (p[i]>>8) | (p[i]<<8);
		//printk(KERN_ERR "DMC(%d):usbdmc_write_eeprom %d $%03x(%4d) \n",pdata->inum,i,a,a);
		res = usb_control_msg(pdata->dev,usb_rcvctrlpipe(pdata->dev,0)
								,IDLE_2_RomWr,0xc0,a,0,s,2,TMV_EEPROM);
		//pktdump("usbdmc_write_eeprom",res,s,pdata->inum);
		if( res <= 0 )
		{
			pdata->errno = res;
			return(-1);
		}
		if( s[0] != ACK )
			break;
	}
	//printk(KERN_ERR "DMC(%d):WRITE %d\n",pdata->inum,i);
	pktdump("usbdmc_write_eeprom",res,s,pdata->inum);
	return( s[0] == ACK ? i : -1 );
}

unsigned long calc_timeup_jiffies(int ms)
{
//	res = (ms * 1000) / (1000000 / HZ)
unsigned long a,b;
	a = ms * 1000;
	b = 1000000 / HZ;
	a = b == 0 ? 1 : (a / b);
	b = jiffies + a;
	//printk(KERN_ERR "DMC:calc_jiffies %u (HZ=%d) \n",a,HZ);
	return(b);
}
#if KER26
static void usbdmc_irq(struct urb *urb, struct pt_regs *regs)
#else
static void usbdmc_irq(struct urb *urb)
#endif
{
int i,getp,putp;
usbdmc_data_t *p;
int inum;
	inum = GET_INTERFACE_NUM(urb->dev);
	#if DEBUG_HON
		printk(KERN_ERR "DMC(%d):usbdmc_irq status %d iProduct=%d\n"
					,inum
					,urb->status
					,urb->dev->descriptor.iProduct);
	#endif
    if( urb->status )
	{
	#if KER26
		goto exit;
	#else
        return;
	#endif
    }
    p = urb->context;
	//pktdump(KERN_ERR "usbdmc_irq",5,p->data,p->inum);
	getp = p->rxgetp;
	putp = p->rxputp;
	if( memcmp(p->svd,p->data,5) == 0 )
	#if KER26
		goto exit;
	#else
		return;
	#endif
	memcpy(p->svd,p->data,5);
	if( p->offdelay == 0 )
	{
		for( i=0 ; i < 5 ; i++ )
		{
			p->rxbuf[putp++] = p->data[i];
			putp = putp >= MAX_RXBUF ? 0 : putp;	
			if( putp == getp )
			#if KER26
				goto exit;
			#else
				return;
			#endif
		}
	}
	else
	{
		if( p->data[0] == 0x10 )
		{
			if( p->offd[0] != 0 )
			{
				printk(KERN_ERR "DMC(%d):usbdmc_irq double off\n",inum);
			#if KER26
				goto exit;
			#else
				return;
			#endif
			}
			p->req_offtimer_timeup = 0;
			memcpy(p->offd,p->data,5);
			init_timer(&p->tmlist);
			p->tmlist.data = (unsigned long)p;
			p->tmlist.function = usbdmc_offtimer;
			p->tmlist.expires = calc_timeup_jiffies(p->offdelay);
			add_timer(&p->tmlist);
		}
		else
		{
			if( p->offd[0] != 0 )
				del_timer(&p->tmlist);
			p->offd[0] = 0;
			for( i=0 ; i < 5 ; i++ )
			{
				p->rxbuf[putp++] = p->data[i];
				putp = putp >= MAX_RXBUF ? 0 : putp;	
				if( putp == getp )
				#if KER26
					goto exit;
				#else
					return;
				#endif
			}
		}
	}
	p->rxputp = putp;
    wake_up_interruptible(&p->wait);
#if KER26
exit:
    p = urb->context;
	usb_submit_urb(p->irq,GFP_ATOMIC);
#endif
}
static void usbdmc_offtimer(unsigned long arg)
{
usbdmc_data_t *p = (usbdmc_data_t *)arg;
int i,putp,getp;
	if( p->offd[0] == 0 )
		goto usbdmc_offtimer_ex;
	getp = p->rxgetp;
	putp = p->rxputp;
	for( i=0 ; i < 5 ; i++ )
	{
		p->rxbuf[putp++] = p->offd[i];
		putp = putp >= MAX_RXBUF ? 0 : putp;	
		if( putp == getp )
			goto usbdmc_offtimer_ex;
	}
	p->offd[0] = 0;
	p->rxputp = putp;
	p->req_offtimer_timeup = 1;
    wake_up_interruptible(&p->wait);
usbdmc_offtimer_ex:
	del_timer(&p->tmlist);
}

static int usbdmc_open(struct inode *inode, struct file *file)
{
usbdmc_data_t *pdata;
int inum;
#if KER26
	inum = iminor(inode) - USBDMC_MINOR;
#else
	inum = MINOR(inode->i_rdev) - USBDMC_MINOR;
#endif
    pdata = inum == 0 ? g_usbdmc_data0 : g_usbdmc_data1;
    if( pdata == NULL )
	{
	#if KER26
		printk(KERN_ERR "DMC(%d):usbdmc_open $%x\n",inum,pdata);
	#endif
        return -ENODEV;
    }
	down(&pdata->sem_entry);
    if( pdata->probe_ok != PROBE_AVAIL )
	{
		up(&pdata->sem_entry);
        return -EBUSY;
	}
#if DEBUG_HON
	printk(KERN_ERR "DMC(%d):usbdmc_open $%x\n",inum,pdata);
#endif
    if( pdata->isopen )
	{
		up(&pdata->sem_entry);
        return -EBUSY;
    }
    init_waitqueue_head(&pdata->wait);
    init_waitqueue_head(&pdata->remove_ok);
    file->private_data = pdata;
    pdata->isopen = 1;
    pdata->rxputp = 0;
    pdata->rxgetp = 0;
#if KER26
    pdata->irq->dev = pdata->dev;
    usb_submit_urb(pdata->irq,GFP_KERNEL);
#else
    pdata->irq.dev = pdata->dev;
    usb_submit_urb(&pdata->irq);
#endif
#if DEBUG_HON
	printk("DMC(%d):usbdmc_open end\n",inum);
#endif
	up(&pdata->sem_entry);
    return 0;
}

static int usbdmc_close(struct inode *inode, struct file *file)
{
usbdmc_data_t *pdata;
    pdata = file->private_data;
#if DEBUG_HON
	printk(KERN_ERR "DMC(%d):usbdmc_close $%x\n",pdata->inum,pdata);
#endif
	down(&pdata->sem_entry);
    if( pdata->isopen )
	{
        pdata->isopen = 0;
	#if KER26
        //usb_unlink_urb(pdata->irq);
        usb_kill_urb(pdata->irq);
	#else
        usb_unlink_urb(&pdata->irq);
	#endif
		//pdata->irq = NULL;
        wake_up(&pdata->remove_ok);
    }
#if DEBUG_HON
	printk("DMC(%d):usbdmc_close end\n",pdata->inum);
#endif
	up(&pdata->sem_entry);
    return 0;
}

#if HAVE_UNLOCKED_IOCTL
static int usbdmc_ioctl(struct file *file,unsigned int cmd, unsigned long arg)
#else
static int usbdmc_ioctl(struct inode *inode, struct file *file,
               unsigned int cmd, unsigned long arg)
#endif
{
usbdmc_data_t* pdata;
int res,a,d;
u_short b;
char buf[512];
int i;
#define	RETRY	5
    pdata = file->private_data;
	#if DEBUG_HON
		printk(KERN_ERR "DMC(%d):usbdmc_ioctl %d($%x)\n",pdata->inum,cmd,cmd);
	#endif
	down(&pdata->sem_entry);
	if( pdata->dev == NULL )
	{
		up(&pdata->sem_entry);
		return(-ENOTCONN);
	}
	res = true;
    switch(cmd)
	{
    case USBDMC_RESET:
		for( i=0 ; i < RETRY ; i++ )
		{
			if( i )
				printk(KERN_ERR "RETRY0(%d)\n",i);
			if( (res = usbdmc_reset(pdata)) )
				break;
			if( pdata->errno == -ENODEV )
				break;
			mdelay(100);
		}
		break;
    case USBDMC_SPECIFY:
		get_user(a,(int *)arg);
		for( i=0 ; i < RETRY ; i++ )
		{
			if( i )
				printk(KERN_ERR "RETRY1(%d)\n",i);
			if( (res = usbdmc_specify(pdata,a)) )
				break;
			if( pdata->errno == -ENODEV )
				break;
			mdelay(100);
		}
		break;
    case USBDMC_GOTO_XYP:
		res = usbdmc_goto_xyp(pdata);
		break;
    case USBDMC_GOTO_IDLE:
		res = usbdmc_goto_idle(pdata);
		break;
    case USBDMC_SETUP:
		res = usbdmc_setup(pdata);
		break;
    case USBDMC_READ_EEPROM:
		for( i=0 ; i < RETRY ; i++ )
		{
			if( i )
				printk(KERN_ERR "RETRY2(%d)\n",i);
			if( (res = usbdmc_read_eeprom(pdata,buf)) > 0 )
			{
				res = copy_to_user((char *)arg,(char *)buf,res);
				res = true;
				break;
			}
			res = false;
			if( pdata->errno == -ENODEV )
				break;
			mdelay(100);
			usbdmc_reset(pdata);
			mdelay(100);
			usbdmc_specify(pdata,pdata->samp_rate);
		}
		if( pdata->errno == -ENODEV )
			break;
		if( i >= RETRY && res == 0 )
			res = true;
		break;
    case USBDMC_WRITE_EEPROM:
		res = copy_from_user((char *)buf,(char *)arg,39*2);
		for( i=0 ; i < RETRY ; i++ )
		{
			if( i )
				printk(KERN_ERR "RETRY3(%d)\n",i);
			if( (res = usbdmc_write_eeprom(pdata,buf)) != -1 )
			{
				res = true;
				break;
			}
			res = false;
			if( pdata->errno == -ENODEV )
				break;
			mdelay(100);
			usbdmc_reset(pdata);
			mdelay(500);
			usbdmc_specify(pdata,pdata->samp_rate);
		}
		break;
    case USBDMC_READ_STAT:
		for( i=0 ; i < RETRY ; i++ )
		{
			if( i )
				printk(KERN_ERR "RETRY4(%d)\n",i);
			if( (res = usbdmc_get_stat(pdata,&b)) )
			{
				put_user(b,(u_short *)arg);
				break;
			}
			if( pdata->errno == -ENODEV )
				break;
			mdelay(100);
		}
		break;
    case USBDMC_READ_MEM:
		get_user(a,(int *)arg);
		for( i=0 ; i < RETRY ; i++ )
		{
			if( i )
				printk(KERN_ERR "RETRY5(%d)\n",i);
			if( (res = usbdmc_readmem(pdata,a,&d)) )
			{
				put_user(d,(int *)arg);
				break;
			}
			if( pdata->errno == -ENODEV )
				break;
			mdelay(100);
		}
		break;
    case USBDMC_WRITE_MEM:
		get_user(a,(int *)arg);
		get_user(d,(int *)arg+1);
		for( i=0 ; i < RETRY ; i++ )
		{
			if( i )
				printk(KERN_ERR "RETRY6(%d)\n",i);
			if( (res = usbdmc_writemem(pdata,a,d)) )
				break;
			if( pdata->errno == -ENODEV )
				break;
			mdelay(100);
		}
		break;
    case USBDMC_SET_OFFDELAY:
		get_user(a,(int *)arg);
		pdata->offdelay = a;
		printk(KERN_ERR "set off-delay = %d\n",pdata->offdelay);
		break;
    case USBDMC_GET_OFFDELAY:
		put_user(pdata->offdelay,(int *)arg);
		printk(KERN_ERR "get off-delay = %d %d\n",pdata->offdelay,res);
		break;
    default:
		up(&pdata->sem_entry);
		return(-ENOIOCTLCMD);
    }
	up(&pdata->sem_entry);
	if( !res )
		return(-EIO);
    return(0);
}

static ssize_t usbdmc_read(struct file * file, char * buffer, size_t count, loff_t *ppos)
{
int i,getp,putp;
usbdmc_data_t *p;
	p = file->private_data;
	#if DEBUG_HON
		printk(KERN_ERR "DMC(%d):usbdmc_read entry size %d\n",p->inum,count);
	#endif
	down(&p->sem_entry);
	getp = p->rxgetp;
	for( i=0 ; i < count ; )
	{
		if( p->dev == NULL )
		{
			printk(KERN_ERR "DMC(%d):usbdmc_read dev-null\n",p->inum);
			up(&p->sem_entry);
			return(-ENOTCONN);
		}
		putp = p->rxputp;
		if( getp == putp )
		{
			if( file->f_flags & O_NONBLOCK )
			{
				//printk(KERN_ERR "DMC(%d):usbdmc_read non-block\n",p->inum);
				if( i > 0 )
					break;
				up(&p->sem_entry);
				return(-EAGAIN);
			}
			up(&p->sem_entry);
			//printk(KERN_ERR "DMC(%d):usbdmc_read irq wait...\n",p->inum);
		#if KER26
			wait_event_interruptible(p->wait,getp!=p->rxputp);
		#else
			interruptible_sleep_on(&p->wait);
		#endif
			//printk(KERN_ERR "DMC(%d):usbdmc_read irq wake\n",p->inum);
			p = file->private_data;
			if( p->dev == NULL )
			{
				printk(KERN_ERR "DMC(%d):usbdmc_read dev-null\n",p->inum);
				return(-ENOTCONN);
			}
			//printk(KERN_ERR "DMC(%d):usbdmc_read irq wakeup\n",p->inum);
			if( signal_pending(current) )
			{
				return(-EINTR);
			}
			down(&p->sem_entry);
		}
		//printk(KERN_ERR "DMC(%d):usbdmc_read data %x\n",p->inum,p->rxbuf[getp]);
		put_user(p->rxbuf[getp++],&buffer[i]);
		getp = getp >= MAX_RXBUF ? 0 : getp;
		i++;
	}
	p->rxgetp = getp;
	#if DEBUG_HON
		printk(KERN_ERR "DMC(%d):usbdmc_read exit %d \n",p->inum,i);
	#endif
	up(&p->sem_entry);
	return(i);
}

static unsigned int usbdmc_poll(struct file *file, poll_table * wwait)
{
usbdmc_data_t *p;
	p = file->private_data;
	#if DEBUG_HON
		printk(KERN_ERR "DMC(%d):usbdmc_poll\n",p->inum);
	#endif
	poll_wait(file,&p->wait,wwait);
	p = file->private_data;
	if( p->dev == NULL )
	{
	#if DEBUG_HON
		printk(KERN_ERR "DMC(%d):usbdmc_poll dev-null\n",p->inum);
	#endif
		return(-ENOTCONN);
	}
	down(&p->sem_entry);
	if( p->rxgetp != p->rxputp )
	{
		up(&p->sem_entry);
		return(POLLIN|POLLRDNORM);
	}
	up(&p->sem_entry);
	return(0);
}

#if KER26
static struct file_operations usbdmc_fileopes = {
#if 0
    .owner=     THIS_MODULE,
#endif
    .read=      usbdmc_read,
    .open=      usbdmc_open,
    .release=   usbdmc_close,
#if HAVE_UNLOCKED_IOCTL
    .unlocked_ioctl=   usbdmc_ioctl,
#else
    .ioctl=   usbdmc_ioctl,
#endif
    .poll=   usbdmc_poll,
};
# if 1
static struct usb_class_driver usbdmc_class0 = {
    .name =     "usb/dmc0",
    .fops =     &usbdmc_fileopes,
    .minor_base =   USBDMC_MINOR,
};
static struct usb_class_driver usbdmc_class1 = {
    .name =     "usb/dmc1",
    .fops =     &usbdmc_fileopes,
    .minor_base =   USBDMC_MINOR+1,
};
# else
static struct usb_class_driver usbdmc_class = {
    .name =     "usb/dmc%d",
    .fops =     &usbdmc_fileopes,
    .minor_base =   USBDMC_MINOR,
};
# endif
#else	/* else not KER26 */
static struct file_operations usbdmc_fileopes = 
{
    owner:          THIS_MODULE,
    ioctl:          usbdmc_ioctl,
    open:           usbdmc_open,
    read:           usbdmc_read,
    poll:           usbdmc_poll,
    release:        usbdmc_close,
};
#endif	/* end not KER26 */

#if KER26
static int usbdmc_probe(struct usb_interface *intface,const struct usb_device_id *id)
{
    struct usb_device *dev = NULL;
    struct usb_host_interface *iface_desc;
    struct usb_endpoint_descriptor *endpoint;
    unsigned int pipe;
    int n, intr_interval, len;
	struct usb_class_driver *class;
	int inum;
	usbdmc_data_t *pdata;
	dev = usb_get_dev(interface_to_usbdev(intface));
	inum = GET_INTERFACE_NUM(dev);
	for( n=0 ; n < 10 ; n++ )
	{
		if( inum == 0 & g_usbdmc_data0 == NULL )
			break;
		if( inum != 0 & g_usbdmc_data1 == NULL )
			break;
		msleep(200);
	}
	if( n >= 10 )
	{
        printk(KERN_INFO "DMC(%d):already use iProduct %d.\n",inum,inum);
        return -EADDRINUSE;
	}
	printk(KERN_ERR "DMC(%d):USB-Driver %s%s\n",inum,TBUSB_VERSION,SUB_VERS);
	printk(KERN_ERR "DMC(%d):usbdmc_probe vendor=$%x product=$%x iProduct=%d\n"
					,inum
					,dev->descriptor.idVendor
					,dev->descriptor.idProduct
					,dev->descriptor.iProduct
					);
	if( dev->descriptor.idVendor != VID_USBDMC )
	{
        printk(KERN_INFO "DMC(%d):unknown vendor id.\n",inum);
        return -ENODEV;
    }
	class = dev->descriptor.iProduct == 0 ? &usbdmc_class0 : &usbdmc_class1;
	iface_desc = intface->cur_altsetting;
	if( inum == 0 )
	{
    	if( !g_usbdmc_data0 )
		{
    		if( (g_usbdmc_data0 = kmalloc(sizeof(usbdmc_data_t)*MAXIF, GFP_KERNEL)) == NULL )
			{
        		printk(KERN_INFO "DMC(%d):no memory\n",inum);
        		return -EIO;
    		}
			memset(g_usbdmc_data0,0,sizeof(usbdmc_data_t));
		}
    	pdata = g_usbdmc_data0;
	}
	else
	{
    	if( !g_usbdmc_data1 )
		{
    		if( (g_usbdmc_data1 = kmalloc(sizeof(usbdmc_data_t)*MAXIF, GFP_KERNEL)) == NULL )
			{
        		printk(KERN_INFO "DMC(%d):no memory\n",inum);
        		return -EIO;
    		}
			memset(g_usbdmc_data1,0,sizeof(usbdmc_data_t));
		}
    	pdata = g_usbdmc_data1;
	}
	sema_init(&pdata->sem_entry,0);
    if( (pdata->irq  = usb_alloc_urb(0, GFP_KERNEL)) == NULL )
	{
		up(&pdata->sem_entry);
        printk(KERN_INFO "DMC(%d):no memory\n",inum);
        return -EIO;
    }
	//memset(pdata->irq,0,sizeof(struct urb));

    intr_interval = 0;
    pdata->ep_intr_in  = -1;
    pdata->ep_bulk_out = -1;
    pdata->ep_bulk_in  = -1;
    for(n = 0; n < iface_desc->desc.bNumEndpoints; n++)
	{
		endpoint = &iface_desc->endpoint[n].desc;
        switch(endpoint->bEndpointAddress&USB_ENDPOINT_NUMBER_MASK)
		{
        case 1: /* INT-IN */
            if( ! (endpoint->bEndpointAddress&USB_ENDPOINT_DIR_MASK) )
                break;
            if( (endpoint->bmAttributes&USB_ENDPOINT_XFERTYPE_MASK) == USB_ENDPOINT_XFER_INT )
			{
                pdata->ep_intr_in = 1;
                intr_interval = endpoint->bInterval;
        		printk("DMC(%d):usbdmc_probe endpoint irq-in 1 ,interval=%d\n",inum,intr_interval);
            }
            break;
        }
    }
    if( pdata->ep_intr_in == -1 )
	{
		up(&pdata->sem_entry);
        printk("endpoint error!!\n");
        return -EIO;
    }
    pdata->dev    = dev;
    pdata->inum   = inum;
	set_mode(pdata,0);
    pdata->isopen = 0;
    pdata->rxputp = 0;
    pdata->rxgetp = 0;
	pdata->offdelay = 0;
	pdata->req_offtimer_timeup = 0;
	pdata->offd[0] = 0;

	usb_set_intfdata(intface,pdata);
	if( usb_register_dev(intface,class) )
	{
		up(&pdata->sem_entry);
		usb_set_intfdata(intface,NULL);
        printk("DMC(%d):regster error!!\n",inum);
        return -EIO;
	}

    pipe = usb_rcvintpipe(pdata->dev, pdata->ep_intr_in);
    len  = usb_maxpacket(dev, pipe, usb_pipeout(pipe));
	//printk("DMC(%d):usb_packet len=%d!!\n",inum,len);
    usb_fill_int_urb(pdata->irq,
                 dev, pipe, pdata->data, len,
                 usbdmc_irq, pdata, intr_interval);
    pdata->irq->transfer_flags = 0;//URB_ISO_ASAP;
    pdata->probe_ok = PROBE_AVAIL;
	up(&pdata->sem_entry);
    return 0;
}

static void usbdmc_disconnect(struct usb_interface *intface)
{
struct usb_class_driver *class;
int inum;
usbdmc_data_t *pdata;
    struct usb_device *dev = NULL;
	dev = usb_get_dev(interface_to_usbdev(intface));
	inum = GET_INTERFACE_NUM(dev);
	class = dev->descriptor.iProduct == 0 ? &usbdmc_class0 : &usbdmc_class1;
	printk(KERN_ERR "DMC(%d):usbdmc_disconnect\n",inum);
	pdata = inum == 0 ? g_usbdmc_data0 : g_usbdmc_data1;
    if( pdata )
	{
		down(&pdata->sem_entry);
        if( pdata->isopen )
		{
			pdata->rxputp = pdata->rxputp+1 >= MAX_RXBUF 
					? 0 : pdata->rxputp+1;	
            pdata->dev = NULL;
            wake_up_interruptible(&pdata->wait);
			up(&pdata->sem_entry);
            wait_event(pdata->remove_ok,pdata->isopen==0);
			//printk(KERN_ERR "DMC(%d):usbdmc_disconnect 1\n",inum);
			down(&pdata->sem_entry);
        }
		usb_set_intfdata(intface,NULL);
    	if( pdata->irq ) 
    		usb_free_urb(pdata->irq);
		usb_deregister_dev(intface,class);
		pdata->probe_ok = PROBE_NONE;
		up(&pdata->sem_entry);
        kfree(pdata);
        if( inum == 0 )
			 g_usbdmc_data0 = NULL;
		else
			 g_usbdmc_data1 = NULL;
    }
	printk(KERN_ERR "DMC(%d):usbdmc_disconnect e\n",inum);
}

#else	// else not KER26
static void* usbdmc_probe(struct usb_device *dev, unsigned int ifnum,
                          const struct usb_device_id *id)
{
    struct usb_interface_descriptor *inf;
    struct usb_config_descriptor    *config;
    unsigned int pipe;
    int n, intr_interval, len;
	int inum;
	usbdmc_data_t *pdata;
	inum = GET_INTERFACE_NUM(dev);
	for( n=0 ; n < 10 ; n++ )
	{
		if( inum == 0 & g_usbdmc_data0 == NULL )
			break;
		if( inum != 0 & g_usbdmc_data1 == NULL )
			break;
		mdelay(200);
	}
	if( n >= 10 )
	{
        printk(KERN_INFO "DMC(%d):already use iProduct %d.\n",inum,inum);
        return -EADDRINUSE;
	}
	printk(KERN_ERR "DMC(%d):USB-Driver %s\n",inum,TBUSB_VERSION);
	printk(KERN_ERR "DMC(%d):usbdmc_probe vendor=$%x product=$%x iProduct=%d\n"
					,inum
					,dev->descriptor.idVendor
					,dev->descriptor.idProduct
					,dev->descriptor.iProduct
					);
	if( dev->descriptor.idVendor != VID_USBDMC )
	{
        printk(KERN_INFO "DMC(%d):USBDMC: unknown vendor id.\n",inum);
        return NULL;
    }
    config = dev->actconfig;
    inf    = &config->interface[ifnum].altsetting[0];
	if( inum == 0 )
	{
    	if( !g_usbdmc_data0 )
		{
    		if( (g_usbdmc_data0 = kmalloc(sizeof(usbdmc_data_t)*MAXIF, GFP_KERNEL)) == NULL )
			{
        		printk(KERN_INFO "DMC(%d):no memory\n",inum);
        		return NULL;
    		}
			memset(g_usbdmc_data0,0,sizeof(usbdmc_data_t));
		}
		pdata = g_usbdmc_data0;
	}
	else
	{
    	if( !g_usbdmc_data1 )
		{
    		if( (g_usbdmc_data1 = kmalloc(sizeof(usbdmc_data_t)*MAXIF, GFP_KERNEL)) == NULL )
			{
        		printk(KERN_INFO "DMC(%d):no memory\n",inum);
        		return NULL;
    		}
			memset(g_usbdmc_data1,0,sizeof(usbdmc_data_t));
		}
		pdata = g_usbdmc_data1;
	}
	sema_init(&pdata->sem_entry,0);
    intr_interval = 0;
    pdata->inum  = inum;
    pdata->ep_intr_in  = -1;
    pdata->ep_bulk_out = -1;
    pdata->ep_bulk_in  = -1;
//printk("DMC(%d):usbdmc_probe 1\n",inum);
    for(n = 0; n < inf->bNumEndpoints; n++)
	{
        switch(inf->endpoint[n].bEndpointAddress&USB_ENDPOINT_NUMBER_MASK)
		{
        case 1: /* INT-IN */
            if( ! (inf->endpoint[n].bEndpointAddress&USB_ENDPOINT_DIR_MASK) )
                break;
            if( (inf->endpoint[n].bmAttributes&USB_ENDPOINT_XFERTYPE_MASK) == USB_ENDPOINT_XFER_INT )
			{
                pdata->ep_intr_in = 1;
                intr_interval = inf->endpoint[n].bInterval;
        		printk("DMC(%d):usbdmc_probe: endpoint irq-in 1 ,interval=%d\n",inum,intr_interval);
            }
            break;
        }
    }
//printk("DMC(%d):usbdmc_probe 2\n",inum);
    if( pdata->ep_intr_in == -1 )
	{
        printk("DMC(%d):endpoint error!!\n",inum);
		up(&pdata->sem_entry);
        return NULL;
    }
    pdata->dev    = dev;
    pdata->mode   = 0;
    pdata->isopen = 0;
    pdata->rxputp = 0;
    pdata->rxgetp = 0;
	pdata->offdelay = 0;
	pdata->req_offtimer_timeup = 0;
	pdata->offd[0] = 0;
//printk("DMC(%d):usbdmc_probe 3\n",inum);

    pipe = usb_rcvintpipe(pdata->dev, pdata->ep_intr_in);
    len  = usb_maxpacket(dev, pipe, usb_pipeout(pipe));
	usb_set_idle(dev, inf->bInterfaceNumber, 0, 0);
    FILL_INT_URB(&pdata->irq,
                 dev, pipe, pdata->data, len,
                 usbdmc_irq, pdata, intr_interval);
#if DEBUG_HON
	printk("DMC(%d):usbdmc_probe end\n",inum);
#endif
    pdata->probe_ok = PROBE_AVAIL;
	up(&pdata->sem_entry);
    return pdata;
}

static void usbdmc_disconnect(struct usb_device *dev, void *ptr)
{
int inum;
usbdmc_data_t *pdata;
	printk(KERN_ERR "usbdmc_disconnect\n");
	inum = GET_INTERFACE_NUM(dev);
	pdata = inum == 0 ? g_usbdmc_data0 : g_usbdmc_data1;
    if( pdata )
	{
		down(&pdata->sem_entry);
        if( pdata->isopen )
		{
			pdata->rxputp = pdata->rxputp+1 >= MAX_RXBUF 
					? 0 : pdata->rxputp+1;	
            pdata->dev = NULL;
            wake_up_interruptible(&pdata->wait);
			up(&pdata->sem_entry);
            sleep_on(&pdata->remove_ok);
			down(&pdata->sem_entry);
			//printk(KERN_ERR "usbdmc_disconnect 1\n");
        }
		pdata->probe_ok = PROBE_NONE;
		up(&pdata->sem_entry);
        kfree(pdata);
		if( inum == 0 )
        	g_usbdmc_data0 = NULL;
		else
        	g_usbdmc_data1 = NULL;
    }
#if DEBUG_HON
	printk(KERN_ERR "usbdmc_disconnect e\n");
#endif
}
#endif	// end not KER26

static struct usb_device_id usbdmc_table [] = 
{
        { USB_DEVICE(VID_USBDMC, PID_USBDMC) },
        { }
};
MODULE_DEVICE_TABLE (usb, usbdmc_table);

#if KER26
static struct usb_driver usbdmc_data = {
#if 0
	.owner =		THIS_MODULE,
#endif
	.name =			"usbdmc",
	.probe =		usbdmc_probe,
	.disconnect =	usbdmc_disconnect,
	.id_table =		usbdmc_table,
};
#else
static struct usb_driver usbdmc_data = 
{
    name:       "usbdmc",
    probe:      usbdmc_probe,
    disconnect: usbdmc_disconnect,
    fops:       &usbdmc_fileopes,
    minor:      USBDMC_MINOR,
    id_table:   usbdmc_table,
};
#endif

int __init usbdmc_init(void)
{
	printk(KERN_ERR "usbdmc_init\n");
    if (usb_register(&usbdmc_data) < 0) 
	{
        return -1;
    }
	printk(KERN_ERR "usbdmc_init ok\n");
    return 0;
}

void __exit usbdmc_exit(void)
{
	printk(KERN_ERR "usbdmc_exit\n");
    usb_deregister(&usbdmc_data);
}

module_init(usbdmc_init);
module_exit(usbdmc_exit);
MODULE_LICENSE("GPL");

